==> iRRAM_2004_01.tar.gz: iRRAM_2004_01/obj/LRGMP/LRMLIB.c <==
#ifndef DEFINITION_Dyadic_Base
#include "LRGMP_Dyadic_Base.h"
#endif

#ifndef DEFINITION_Dyadic
#include "LRGMP_Dyadic.h"
#endif

#ifndef DEFINITION_LRMLIB
#include "LRGMP_lib.h"
#endif

#define   FALSE	0
#define   TRUE	1

static Dyadic_Dyadic PIviertel;
static long PIprec;
static Dyadic_Dyadic EULER;
static long EULERprec;
static void LREXP1 (Dyadic_Dyadic *X, Dyadic_Dyadic *Z, long N);
static void LRSINUS1 (Dyadic_Dyadic *X, Dyadic_Dyadic *Z, long N);
static void LRCOSINUS1 (Dyadic_Dyadic *X, Dyadic_Dyadic *Z, long N);
static void INIT ();


void LRMLIB_LREXP (Dyadic_Dyadic *X, Dyadic_Dyadic *Z, long N)
{
  Dyadic_Dyadic X1, Z1;
  long i;

  Dyadic_LRINIT(&X1);
  Dyadic_LRINIT(&Z1);
  Dyadic_INTTOLR(1, &Z1);
  Dyadic_INTTOLR(1, &X1);
  i = 0;
  for (;;) {
    i++;
    Dyadic_LRDIVINT(&X1, i, &X1, N - 100);
    Dyadic_LRMULTKARAT(&X1, X, &X1, N - 100);
    Dyadic_LRADD(&Z1, &X1, &Z1, N - 100);
    if (Dyadic_LRSIGN(&X1) == 0) {
      goto EXIT_1;
    }
    if (Dyadic_LRSIZE(&X1) < N - 5) {
      goto EXIT_1;
    }
  } EXIT_1:;
  Dyadic_LRADD(&Z1, &X1, Z, N - 100);
  Dyadic_LRCLEAR(&Z1);
  Dyadic_LRCLEAR(&X1);
}

static void LREXP1(Dyadic_Dyadic *X, Dyadic_Dyadic *Z, long N)
{
  Dyadic_Dyadic X1, Z1;
  long i;

  Dyadic_LRINIT(&X1);
  Dyadic_LRINIT(&Z1);
  Dyadic_INTTOLR(1, &Z1);
  Dyadic_INTTOLR(1, &X1);
  i = 0;
  for (;;) {
    i++;
    Dyadic_LRDIVINT(&X1, i, &X1, N - 5);
    Dyadic_LRMULTKARAT(&X1, X, &X1, N - 5);
    Dyadic_LRADD(&Z1, &X1, &Z1, N - 5);
    if (Dyadic_LRSIZE(&X1) < N - 5) {
      goto EXIT_2;
    }
  } EXIT_2:;
  Dyadic_LRADD(&Z1, &X1, Z, N - 100);
  Dyadic_LRCLEAR(&Z1);
  Dyadic_LRCLEAR(&X1);
}

static void LRSINUS1(Dyadic_Dyadic *X, Dyadic_Dyadic *Z, long N)
{
  Dyadic_Dyadic X2, X1;
  long i;
  long p;

  p = 2*Dyadic_LOG2(N)+10;

  Dyadic_LRINIT(&X1);
  Dyadic_LRINIT(&X2);
  Dyadic_LRMULTKARAT(X, X, &X2, N - p);
  Dyadic_LRCOPY(X, &X1, N - p);
  Dyadic_LRCOPY(X, Z, N - p);
  i = 2;
  while (Dyadic_LRSIZE(&X1) > N + 1) {
    Dyadic_LRDIVINT(&X1, -i, &X1, N - p);
    Dyadic_LRDIVINT(&X1, i + 1, &X1, N - p);
    Dyadic_LRMULTKARAT(&X1, &X2, &X1, N - p);
    Dyadic_LRADD(Z, &X1, Z, N - p);
    i += 2;
  }
  Dyadic_LRCLEAR(&X1);
  Dyadic_LRCLEAR(&X2);
}

static void LRCOSINUS1 (Dyadic_Dyadic *X, Dyadic_Dyadic *Z, long N)
{
  Dyadic_Dyadic X2, X1;
  long i;
  long p;

  p = 2*Dyadic_LOG2(N)+10;

  Dyadic_LRINIT(&X1);
  Dyadic_LRINIT(&X2);
  Dyadic_LRMULTKARAT(X, X, &X2, N - p);
  Dyadic_INTTOLR(1, Z);
  Dyadic_INTTOLR(1, &X1);
  i = 1;
  while (Dyadic_LRSIZE(&X1) > N) {
    Dyadic_LRDIVINT(&X1, -i, &X1, N - p);
    Dyadic_LRDIVINT(&X1, i + 1, &X1, N - p);
    Dyadic_LRMULTKARAT(&X1, &X2, &X1, N - p);
    Dyadic_LRADD(Z, &X1, Z, N - p);
    i += 2 ;
  }
  Dyadic_LRCLEAR(&X1);
  Dyadic_LRCLEAR(&X2);
}

void LRMLIB_LRCOSINUS (Dyadic_Dyadic *X, Dyadic_Dyadic *Z, long N)
{
  Dyadic_Dyadic PI, X1, Z1, Y1;
  long xint;
  long s;

  if ( N >= 0) {
    Dyadic_INTTOLR(0,Z);
  } else {
  s = Dyadic_LRSIZE(X);
  if (s <= 0) {
    LRCOSINUS1(X, Z, N);
  } else {
    Dyadic_LRINIT(&X1);
    Dyadic_LRINIT(&Y1);
    Dyadic_LRINIT(&Z1);
    Dyadic_LRINIT(&PI);
    LRMLIB_LRPI(&PI, N - s - 10);
    Dyadic_LRMULTINT(&PI, 2, &Z1, N - s - 10);
    Dyadic_LRDIV(X, &Z1, &X1, N - 7);
    Dyadic_LRFRAC(&X1, &X1);
    Dyadic_LRMULTKARAT(&X1, &Z1, &X1, N - 4);
    Dyadic_LRMULTINT(&X1, 100, &Y1, 0);
    Dyadic_LRTOINT(&Y1, &xint);
    if (xint < 80) {
      LRCOSINUS1(&X1, Z, N - 1);
    } else if (xint < 235) {
      Dyadic_LRDIVINT(&PI, 2, &Z1, N - 4);
      Dyadic_LRSUB(&X1, &Z1, &X1, N - 2);
      LRSINUS1(&X1, Z, N - 1);
      Dyadic_LRNEGATE(Z);
    } else if (xint < 392) {
      Dyadic_LRSUB(&X1, &PI, &X1, N - 2);
      LRCOSINUS1(&X1, Z, N - 1);
      Dyadic_LRNEGATE(Z);
    } else if (xint < 550) {
      Dyadic_LRMULTINT(&PI, 3, &Z1, N - 4);
      Dyadic_LRDIVINT(&Z1, 2, &Z1, N - 4);
      Dyadic_LRSUB(&X1, &Z1, &X1, N - 2);
      LRSINUS1(&X1, Z, N - 1);
    } else {
      Dyadic_LRSUB(&X1, &Z1, &X1, N - 2);
      LRCOSINUS1(&X1, Z, N - 1);
    }
    Dyadic_LRCLEAR(&PI);
    Dyadic_LRCLEAR(&Z1);
    Dyadic_LRCLEAR(&Y1);
    Dyadic_LRCLEAR(&X1);
  }
  }
}

void LRMLIB_LRSINUS (Dyadic_Dyadic *X, Dyadic_Dyadic *Z, long N)
{
  Dyadic_Dyadic PI, X1, Z1, Y1;
  BOOLEAN neg;
  long xint;
  long s;

  if ( N >= 0) {
    Dyadic_INTTOLR(0,Z);
  } else {
  s = Dyadic_LRSIZE(X);
  if (s <= 0) {
    LRSINUS1(X, Z, N);
  } else {
    Dyadic_LRINIT(&X1);
    Dyadic_LRINIT(&Y1);
    Dyadic_LRINIT(&Z1);
    Dyadic_LRINIT(&PI);
    LRMLIB_LRPI(&PI, N - s - 10);
    Dyadic_LRMULTINT(&PI, 2, &Z1, N - s - 10);
    Dyadic_LRDIV(X, &Z1, &X1, N - 7);
    Dyadic_LRFRAC(&X1, &X1);
    Dyadic_LRMULTKARAT(&X1, &Z1, &X1, N - 4);
    if (Dyadic_LRSIGN(&X1) == -1) {
      Dyadic_LRNEGATE(&X1);
      neg = TRUE;
    } else {
      neg = FALSE;
    }
    Dyadic_LRMULTINT(&X1, 100, &Y1, 0);
    Dyadic_LRTOINT(&Y1, &xint);
    if (xint < 80) {
      LRSINUS1(&X1, Z, N - 1);
    } else if (xint < 235) {
      Dyadic_LRDIVINT(&PI, 2, &Z1, N - 4);
      Dyadic_LRSUB(&X1, &Z1, &X1, N - 2);
      LRCOSINUS1(&X1, Z, N - 1);
    } else if (xint < 392) {
      Dyadic_LRSUB(&X1, &PI, &X1, N - 2);
      LRSINUS1(&X1, Z, N - 1);
      Dyadic_LRNEGATE(Z);
    } else if (xint < 550) {
      Dyadic_LRMULTINT(&PI, 3, &Z1, N - 4);
      Dyadic_LRDIVINT(&Z1, 2, &Z1, N - 4);
      Dyadic_LRSUB(&X1, &Z1, &X1, N - 2);
      LRCOSINUS1(&X1, Z, N - 1);
      Dyadic_LRNEGATE(Z);
    } else {
      Dyadic_LRSUB(&X1, &Z1, &X1, N - 2);
      LRSINUS1(&X1, Z, N - 1);
    }
    if (neg) {
      Dyadic_LRNEGATE(Z);
    }
    Dyadic_LRCLEAR(&PI);
    Dyadic_LRCLEAR(&Z1);
    Dyadic_LRCLEAR(&Y1);
    Dyadic_LRCLEAR(&X1);
  }
  }
}

void LRMLIB_LRPI (Dyadic_Dyadic *Z, long N)
{
  long i, p;
  Dyadic_Dyadic X, Y;

  if (N <= PIprec) {
    Dyadic_LRINIT(&X);
    Dyadic_LRINIT(&Y);
    Dyadic_INTTOLR(0, &PIviertel);
    p = Dyadic_LOG2(-N);
    Dyadic_INTTOLR(4, &X);
    i = 1;
    Dyadic_LRDIVINT(&X, 5, &X, N - p);
    Dyadic_LRADD(&PIviertel, &X, &PIviertel, N - p);
    for (;;) {
      if (Dyadic_LRSIGN(&X) == 0) {
        goto EXIT_3;
      }
      if (Dyadic_LRSIZE(&X) < N - 1) {
        goto EXIT_3;
      }
      Dyadic_LRDIVINT(&X, -25, &X, N - p);
      i += 2;
      Dyadic_LRDIVINT(&X, i, &Y, N - p);
      Dyadic_LRADD(&PIviertel, &Y, &PIviertel, N - p);
    } EXIT_3:;
    Dyadic_INTTOLR(1, &X);
    i = 1;
    Dyadic_LRDIVINT(&X, 239, &X, N - p);
    Dyadic_LRSUB(&PIviertel, &X, &PIviertel, N - p);
    for (;;) {
      if (Dyadic_LRSIGN(&X) == 0) {
        goto EXIT_4;
      }
      if (Dyadic_LRSIZE(&X) < N - 1) {
        goto EXIT_4;
      }
      Dyadic_LRDIVINT(&X, -239, &X, N - p);
      Dyadic_LRDIVINT(&X, 239, &X, N - p);
      i += 2;
      Dyadic_LRDIVINT(&X, i, &Y, N - p);
      Dyadic_LRSUB(&PIviertel, &Y, &PIviertel, N - p);
    } EXIT_4:;
    PIprec = N;
    Dyadic_LRCLEAR(&X);
    Dyadic_LRCLEAR(&Y);
  }
  Dyadic_LRMULTINT(&PIviertel, 4, Z, N - 1);
}

void LRMLIB_LREULER (Dyadic_Dyadic *Z, long N)
{
  long i, p;
  Dyadic_Dyadic X;

  if (N <= EULERprec) {
    Dyadic_LRINIT(&X);
    Dyadic_INTTOLR(2, &EULER);
    p = Dyadic_LOG2(-N);
    Dyadic_INTTOLR(1, &X);
    i = 2;
    for (;;) {
      Dyadic_LRDIVINT(&X, i, &X, N - p);
      Dyadic_LRADD(&EULER, &X, &EULER, N - p);
      if (Dyadic_LRSIGN(&X) == 0) {
        goto EXIT_5;
      }
      if (Dyadic_LRSIZE(&X) < N - 1) {
        goto EXIT_5;
      }
      i += 1;
    } EXIT_5:;
    EULERprec = N;
    Dyadic_LRCLEAR(&X);
  }
  Dyadic_LRCOPY(&EULER, Z, N - 1);
}

static void INIT ()
{
  Dyadic_Dyadic X;

  Dyadic_LRINIT(&X);
  Dyadic_LRINIT(&EULER);
  Dyadic_INTTOLR(3, &EULER);
  EULERprec = 0;
  LRMLIB_LREULER(&X, -100);
  Dyadic_LRINIT(&PIviertel);
  Dyadic_INTTOLR(1, &PIviertel);
  PIprec = 0;
  LRMLIB_LRPI(&X, -100);
  Dyadic_LRCLEAR(&X);
}

void BEGIN_LRMLIB()
{
  static BOOLEAN has_been_called = FALSE;

  if (!has_been_called) {
    has_been_called = TRUE;

    BEGIN_Dyadic();

    INIT();
  }
}

==> iRRAM_2005_01.tar.gz: iRRAM_2005_01/obj/LRGMP/LRMLIB.c <==
#ifndef DEFINITION_Dyadic_Base
#include "LRGMP_Dyadic_Base.h"
#endif

#ifndef DEFINITION_Dyadic
#include "LRGMP_Dyadic.h"
#endif

#ifndef DEFINITION_LRMLIB
#include "LRGMP_lib.h"
#endif

#define   FALSE	0
#define   TRUE	1

static Dyadic_Dyadic PIviertel;
static int PIprec;
static Dyadic_Dyadic EULER;
static int EULERprec;
static void LREXP1 (Dyadic_Dyadic *X, Dyadic_Dyadic *Z, int N);
static void LRSINUS1 (Dyadic_Dyadic *X, Dyadic_Dyadic *Z, int N);
static void LRCOSINUS1 (Dyadic_Dyadic *X, Dyadic_Dyadic *Z, int N);
static void INIT ();


void LRMLIB_LREXP (Dyadic_Dyadic *X, Dyadic_Dyadic *Z, int N)
{
  Dyadic_Dyadic X1, Z1;
  int i;

  Dyadic_LRINIT(&X1);
  Dyadic_LRINIT(&Z1);
  Dyadic_INTTOLR(1, &Z1);
  Dyadic_INTTOLR(1, &X1);
  i = 0;
  for (;;) {
    i++;
    Dyadic_LRDIVINT(&X1, i, &X1, N - 100);
    Dyadic_LRMULTKARAT(&X1, X, &X1, N - 100);
    Dyadic_LRADD(&Z1, &X1, &Z1, N - 100);
    if (Dyadic_LRSIGN(&X1) == 0) {
      goto EXIT_1;
    }
    if (Dyadic_LRSIZE(&X1) < N - 5) {
      goto EXIT_1;
    }
  } EXIT_1:;
  Dyadic_LRADD(&Z1, &X1, Z, N - 100);
  Dyadic_LRCLEAR(&Z1);
  Dyadic_LRCLEAR(&X1);
}

static void LREXP1(Dyadic_Dyadic *X, Dyadic_Dyadic *Z, int N)
{
  Dyadic_Dyadic X1, Z1;
  int i;

  Dyadic_LRINIT(&X1);
  Dyadic_LRINIT(&Z1);
  Dyadic_INTTOLR(1, &Z1);
  Dyadic_INTTOLR(1, &X1);
  i = 0;
  for (;;) {
    i++;
    Dyadic_LRDIVINT(&X1, i, &X1, N - 5);
    Dyadic_LRMULTKARAT(&X1, X, &X1, N - 5);
    Dyadic_LRADD(&Z1, &X1, &Z1, N - 5);
    if (Dyadic_LRSIZE(&X1) < N - 5) {
      goto EXIT_2;
    }
  } EXIT_2:;
  Dyadic_LRADD(&Z1, &X1, Z, N - 100);
  Dyadic_LRCLEAR(&Z1);
  Dyadic_LRCLEAR(&X1);
}

static void LRSINUS1(Dyadic_Dyadic *X, Dyadic_Dyadic *Z, int N)
{
  Dyadic_Dyadic X2, X1;
  int i;
  int p;

  p = 2*Dyadic_LOG2(N)+10;

  Dyadic_LRINIT(&X1);
  Dyadic_LRINIT(&X2);
  Dyadic_LRMULTKARAT(X, X, &X2, N - p);
  Dyadic_LRCOPY(X, &X1, N - p);
  Dyadic_LRCOPY(X, Z, N - p);
  i = 2;
  while (Dyadic_LRSIZE(&X1) > N + 1) {
    Dyadic_LRDIVINT(&X1, -i, &X1, N - p);
    Dyadic_LRDIVINT(&X1, i + 1, &X1, N - p);
    Dyadic_LRMULTKARAT(&X1, &X2, &X1, N - p);
    Dyadic_LRADD(Z, &X1, Z, N - p);
    i += 2;
  }
  Dyadic_LRCLEAR(&X1);
  Dyadic_LRCLEAR(&X2);
}

static void LRCOSINUS1 (Dyadic_Dyadic *X, Dyadic_Dyadic *Z, int N)
{
  Dyadic_Dyadic X2, X1;
  int i;
  int p;

  p = 2*Dyadic_LOG2(N)+10;

  Dyadic_LRINIT(&X1);
  Dyadic_LRINIT(&X2);
  Dyadic_LRMULTKARAT(X, X, &X2, N - p);
  Dyadic_INTTOLR(1, Z);
  Dyadic_INTTOLR(1, &X1);
  i = 1;
  while (Dyadic_LRSIZE(&X1) > N) {
    Dyadic_LRDIVINT(&X1, -i, &X1, N - p);
    Dyadic_LRDIVINT(&X1, i + 1, &X1, N - p);
    Dyadic_LRMULTKARAT(&X1, &X2, &X1, N - p);
    Dyadic_LRADD(Z, &X1, Z, N - p);
    i += 2 ;
  }
  Dyadic_LRCLEAR(&X1);
  Dyadic_LRCLEAR(&X2);
}

void LRMLIB_LRCOSINUS (Dyadic_Dyadic *X, Dyadic_Dyadic *Z, int N)
{
  Dyadic_Dyadic PI, X1, Z1, Y1;
  int xint;
  int s;

  if ( N >= 0) {
    Dyadic_INTTOLR(0,Z);
  } else {
  s = Dyadic_LRSIZE(X);
  if (s <= 0) {
    LRCOSINUS1(X, Z, N);
  } else {
    Dyadic_LRINIT(&X1);
    Dyadic_LRINIT(&Y1);
    Dyadic_LRINIT(&Z1);
    Dyadic_LRINIT(&PI);
    LRMLIB_LRPI(&PI, N - s - 10);
    Dyadic_LRMULTINT(&PI, 2, &Z1, N - s - 10);
    Dyadic_LRDIV(X, &Z1, &X1, N - 7);
    Dyadic_LRFRAC(&X1, &X1);
    Dyadic_LRMULTKARAT(&X1, &Z1, &X1, N - 4);
    Dyadic_LRMULTINT(&X1, 100, &Y1, 0);
    Dyadic_LRTOINT(&Y1, &xint);
    if (xint < 80) {
      LRCOSINUS1(&X1, Z, N - 1);
    } else if (xint < 235) {
      Dyadic_LRDIVINT(&PI, 2, &Z1, N - 4);
      Dyadic_LRSUB(&X1, &Z1, &X1, N - 2);
      LRSINUS1(&X1, Z, N - 1);
      Dyadic_LRNEGATE(Z);
    } else if (xint < 392) {
      Dyadic_LRSUB(&X1, &PI, &X1, N - 2);
      LRCOSINUS1(&X1, Z, N - 1);
      Dyadic_LRNEGATE(Z);
    } else if (xint < 550) {
      Dyadic_LRMULTINT(&PI, 3, &Z1, N - 4);
      Dyadic_LRDIVINT(&Z1, 2, &Z1, N - 4);
      Dyadic_LRSUB(&X1, &Z1, &X1, N - 2);
      LRSINUS1(&X1, Z, N - 1);
    } else {
      Dyadic_LRSUB(&X1, &Z1, &X1, N - 2);
      LRCOSINUS1(&X1, Z, N - 1);
    }
    Dyadic_LRCLEAR(&PI);
    Dyadic_LRCLEAR(&Z1);
    Dyadic_LRCLEAR(&Y1);
    Dyadic_LRCLEAR(&X1);
  }
  }
}

void LRMLIB_LRSINUS (Dyadic_Dyadic *X, Dyadic_Dyadic *Z, int N)
{
  Dyadic_Dyadic PI, X1, Z1, Y1;
  BOOLEAN neg;
  int xint;
  int s;

  if ( N >= 0) {
    Dyadic_INTTOLR(0,Z);
  } else {
  s = Dyadic_LRSIZE(X);
  if (s <= 0) {
    LRSINUS1(X, Z, N);
  } else {
    Dyadic_LRINIT(&X1);
    Dyadic_LRINIT(&Y1);
    Dyadic_LRINIT(&Z1);
    Dyadic_LRINIT(&PI);
    LRMLIB_LRPI(&PI, N - s - 10);
    Dyadic_LRMULTINT(&PI, 2, &Z1, N - s - 10);
    Dyadic_LRDIV(X, &Z1, &X1, N - 7);
    Dyadic_LRFRAC(&X1, &X1);
    Dyadic_LRMULTKARAT(&X1, &Z1, &X1, N - 4);
    if (Dyadic_LRSIGN(&X1) == -1) {
      Dyadic_LRNEGATE(&X1);
      neg = TRUE;
    } else {
      neg = FALSE;
    }
    Dyadic_LRMULTINT(&X1, 100, &Y1, 0);
    Dyadic_LRTOINT(&Y1, &xint);
    if (xint < 80) {
      LRSINUS1(&X1, Z, N - 1);
    } else if (xint < 235) {
      Dyadic_LRDIVINT(&PI, 2, &Z1, N - 4);
      Dyadic_LRSUB(&X1, &Z1, &X1, N - 2);
      LRCOSINUS1(&X1, Z, N - 1);
    } else if (xint < 392) {
      Dyadic_LRSUB(&X1, &PI, &X1, N - 2);
      LRSINUS1(&X1, Z, N - 1);
      Dyadic_LRNEGATE(Z);
    } else if (xint < 550) {
      Dyadic_LRMULTINT(&PI, 3, &Z1, N - 4);
      Dyadic_LRDIVINT(&Z1, 2, &Z1, N - 4);
      Dyadic_LRSUB(&X1, &Z1, &X1, N - 2);
      LRCOSINUS1(&X1, Z, N - 1);
      Dyadic_LRNEGATE(Z);
    } else {
      Dyadic_LRSUB(&X1, &Z1, &X1, N - 2);
      LRSINUS1(&X1, Z, N - 1);
    }
    if (neg) {
      Dyadic_LRNEGATE(Z);
    }
    Dyadic_LRCLEAR(&PI);
    Dyadic_LRCLEAR(&Z1);
    Dyadic_LRCLEAR(&Y1);
    Dyadic_LRCLEAR(&X1);
  }
  }
}

void LRMLIB_LRPI (Dyadic_Dyadic *Z, int N)
{
  int i, p;
  Dyadic_Dyadic X, Y;

  if (N <= PIprec) {
    Dyadic_LRINIT(&X);
    Dyadic_LRINIT(&Y);
    Dyadic_INTTOLR(0, &PIviertel);
    p = Dyadic_LOG2(-N);
    Dyadic_INTTOLR(4, &X);
    i = 1;
    Dyadic_LRDIVINT(&X, 5, &X, N - p);
    Dyadic_LRADD(&PIviertel, &X, &PIviertel, N - p);
    for (;;) {
      if (Dyadic_LRSIGN(&X) == 0) {
        goto EXIT_3;
      }
      if (Dyadic_LRSIZE(&X) < N - 1) {
        goto EXIT_3;
      }
      Dyadic_LRDIVINT(&X, -25, &X, N - p);
      i += 2;
      Dyadic_LRDIVINT(&X, i, &Y, N - p);
      Dyadic_LRADD(&PIviertel, &Y, &PIviertel, N - p);
    } EXIT_3:;
    Dyadic_INTTOLR(1, &X);
    i = 1;
    Dyadic_LRDIVINT(&X, 239, &X, N - p);
    Dyadic_LRSUB(&PIviertel, &X, &PIviertel, N - p);
    for (;;) {
      if (Dyadic_LRSIGN(&X) == 0) {
        goto EXIT_4;
      }
      if (Dyadic_LRSIZE(&X) < N - 1) {
        goto EXIT_4;
      }
      Dyadic_LRDIVINT(&X, -239, &X, N - p);
      Dyadic_LRDIVINT(&X, 239, &X, N - p);
      i += 2;
      Dyadic_LRDIVINT(&X, i, &Y, N - p);
      Dyadic_LRSUB(&PIviertel, &Y, &PIviertel, N - p);
    } EXIT_4:;
    PIprec = N;
    Dyadic_LRCLEAR(&X);
    Dyadic_LRCLEAR(&Y);
  }
  Dyadic_LRMULTINT(&PIviertel, 4, Z, N - 1);
}

void LRMLIB_LREULER (Dyadic_Dyadic *Z, int N)
{
  int i, p;
  Dyadic_Dyadic X;

  if (N <= EULERprec) {
    Dyadic_LRINIT(&X);
    Dyadic_INTTOLR(2, &EULER);
    p = Dyadic_LOG2(-N);
    Dyadic_INTTOLR(1, &X);
    i = 2;
    for (;;) {
      Dyadic_LRDIVINT(&X, i, &X, N - p);
      Dyadic_LRADD(&EULER, &X, &EULER, N - p);
      if (Dyadic_LRSIGN(&X) == 0) {
        goto EXIT_5;
      }
      if (Dyadic_LRSIZE(&X) < N - 1) {
        goto EXIT_5;
      }
      i += 1;
    } EXIT_5:;
    EULERprec = N;
    Dyadic_LRCLEAR(&X);
  }
  Dyadic_LRCOPY(&EULER, Z, N - 1);
}

static void INIT ()
{
  Dyadic_Dyadic X;

  Dyadic_LRINIT(&X);
  Dyadic_LRINIT(&EULER);
  Dyadic_INTTOLR(3, &EULER);
  EULERprec = 0;
  LRMLIB_LREULER(&X, -100);
  Dyadic_LRINIT(&PIviertel);
  Dyadic_INTTOLR(1, &PIviertel);
  PIprec = 0;
  LRMLIB_LRPI(&X, -100);
  Dyadic_LRCLEAR(&X);
}

void BEGIN_LRMLIB()
{
  static BOOLEAN has_been_called = FALSE;

  if (!has_been_called) {
    has_been_called = TRUE;

    BEGIN_Dyadic();

    INIT();
  }
}
